﻿/* 

Скрипт расставляет оригиналы на место превьюшек при условии что у превью и оригинала в имени файла есть один и тот же идентификатор.
Идентификатор - это набор цифр от 5 и более.

В папке с оригиналами могут быть ещё папки с фотографиями, скрипт достанет их все из подпапок.
Скрипт ловит форматы jpg, jpeg, psd, tif, tiff, png, bmp, gif, jfif, eps, ai.
Алгоритм упрощен по сравнению со старым скриптом ShutterHand.jsx.

Скрипт больше не ориентируется на слова "Depositphotos" или "Shutterstock".
С одной стороны есть ооооочень маленькая вероятность перелинковать что-то не то (для этого должны совпасть все 5+ цифр идентификатора).
С другой стороны он работает независимо от того какие мы используем фотобанки и от того, как они в очередной раз по-новому решили называть файлы
Я бы не волновался, но запускал скрипт поразворотно, а не на весь документ сразу.

Регулярное выражение для идентификатора хранится в переменной idRegExp.

Алексей Стукун, st_verstka@mail.ru.

*/

app.doScript(main, undefined , undefined,UndoModes.fastEntireScript,"Run " + app.activeScript.name);

function main() { 
    
    app.scriptPreferences.userInteractionLevel = UserInteractionLevels.INTERACT_WITH_ALL; 
    
    if (app.documents.length == 0) {
        alert("No documents are open");
        return;
    }

    File.prototype.stockID = null;
    // "\d" – цифра, число с запятой – минимальное количество цифр подряд и более
    var idRegExp = /\d{5,}/; 
    var myDoc = app.activeDocument;
    var errLog = [];
    var succLog = 0;

    // Показываем меню:
    myDialog = app.dialogs.add({ name: "Shutter Gun Options:" });
    with (myDialog.dialogColumns.add()) {
        var RB = radiobuttonGroups.add();
        with (RB) {
            var RBDoc = radiobuttonControls.add({ staticLabel: "Document" });
            var RBSpread = radiobuttonControls.add({ staticLabel: "Spread", checkedState: true });             
            if (app.selection.length>0){
                var RBSelect = radiobuttonControls.add({ staticLabel: "Selection", checkedState: true });
                }
            staticTexts.add(); 
            var errLogBox = dialogRows.add().checkboxControls.add( {staticLabel: "Show report", checkedState: true} );
            }
    }
    var myResult = myDialog.show();
    if (!myResult) { return }

    // Собираем картинки в папке:
    var picFolder = chooseFolder("Choose Folder");
    if (picFolder == null) { return }
    var myFiles = [];
    var picFiles = getSubFoldersFiles(picFolder, /.+\.(?:jpg|jpeg|psd|tif|tiff|png|bmp|gif|jfif|eps|ai)$/i); 
    
    // Присваиваем каждой картинке в папке stockID:
    for (i = 0, iLen = picFiles.length; i < iLen; i++) {
        var picFile = picFiles[i];
        myName = picFile.fullName;
        myName = myName.substring(myName.lastIndexOf("/"));
        myName = myName.substring(1);
        myName = myName.match(idRegExp);
        if (myName === null) { continue }
        picFile.stockID = myName[0];
    }

    // Собираем картинки в документе:
    switch (RB.selectedButton) {
        case 0:
        var myImages = myDoc.allGraphics;
        break;
        case 1:
        var myImages = app.activeWindow.activeSpread.allGraphics;
        break;
        case 2:
        var mySelection = app.selection;
        var myImages = new Array ();
        for (s=0; s<(mySelection.length); s++) {
            var myImages = myImages.concat(mySelection[s].allGraphics);
        }
    }

    // И для каждой подходящей картинки ищем оригинал:
    for (i = 0, iLen = myImages.length; i < iLen; i++) {
        
        var myLink = myImages[i].itemLink;
        var myFrame = myImages[i].parent;
        var myName = myLink.name;
        var myPath = myLink.filePath;
        
        // Исключаем из обработки:
        if (!myFrame.parentPage){ continue; } // На полях
        if (myLink.status == 1282237028) { continue; } // Embegged
        if (myFrame.locked == true) { continue; } // Заблокированные объекты
        if (myFrame.itemLayer.locked == true) { continue; } // Заблокированные слои
        if (myFrame.itemLayer.visible == false) { continue; } // Невидимые слои
        if (myLink.linkType != "JPEG") { continue; } // Не JPG
        
        // Находим ID:
        var linkID = myLink.name.match(idRegExp);
        if (linkID == null) { continue }
        linkID = linkID[0];
        
        // Ищем картинку с тем же ID:
        if (findAndRelink(linkID, picFiles)) {
            continue;
        } else {
            if (errLogBox.checkedState) {
                errLog.push("page " + myFrame.parentPage.name + ":\r" + myName);
            }
        }


    }

    // Показываем лог:
    if (errLog.length > 0) {
        errLog.sort( function(a, b) { return a.slice(a.indexOf(" "), a.indexOf(":")) - b.slice(b.indexOf(" "), b.indexOf(":")); });    
        alertScroll("Done!", "RELINKED: " + succLog + " links.\rFAILED: " + errLog.length + " links:\r" + errLog);
        } else {
        alert ("Relinked items: " + succLog);
    }

    // ==================================================== ФУНКЦИИ: ====================================================

    function findAndRelink(linkID, picFiles) {
        for (p = 0, pLen = picFiles.length; p < pLen; p++) {
            var picFile = picFiles[p];
            if (picFile.stockID == linkID) {
                try {
                    myLink.relink(picFile);
                    ++succLog;
                    return true;
                    } catch (error) {
                }
            }
        }
        return false;
    }

    function alertScroll (title, input) {
        if (input instanceof Array)
        input = input.join ("\r"); 
        var myWindow =    new Window ("dialog", title);
        var myList = myWindow.add ("edittext", undefined, input, {multiline: true, scrolling: true});
        myList.maximumSize.height = 700;
        myList.maximumSize.width = myWindow.maximumSize.width;
        myList.minimumSize.height = 10;
        myList.minimumSize.width = 1200;
        myWindow.add ("button", undefined, "Ok", {name: "Ok"});
        myWindow.show();
    }

    function getSubFoldersFiles(myFolder, myMask) {  
                var myFileList = myFolder.getFiles();
                for (var i = 0, iLen = myFileList.length; i < iLen;  i++) {
                    var myFile = myFileList[i];  
                    if (myFile instanceof Folder) { 
                        getSubFoldersFiles(myFile, myMask);  
                    }  
                    if (myFile instanceof File && myFile.name.match(myMask)) {  
                        myFiles.push(myFile);  
                    }
                } 
                return myFiles;
    }

    function chooseFolder(title) {
            var myFolder = Folder.selectDialog(title);
            if (myFolder == null) { exit(); }
            // Получаем все файлы:
            // var myAllFiles = myFolder.getFiles();
            return myFolder;
    }


}